module emission
  !-----------------------------------------------------------------------
  !       purpose
  !       -------
  !       perform emissions needed for TM5 CBM4 version
  !
  !       interface
  !       ---------
  !       call declare_emission
  !       call emission_apply
  !       call free_emission
  !
  !       method
  !       ------
  !       subroutine declare_emission is called from trace0
  !       subroutine emission_apply   is called from source1
  !       subroutine free_emission    is called from trace_end
  !-----------------------------------------------------------------------
  use emission_data,  only: budemi_dat, plandr, emis2D
  use emission_data,  only: sum_emission
  use emission_co2_all  , only: declare_emission_co2_all    , free_emission_co2_all    , emission_apply_co2_all  , calc_emission_co2_all
  use emission_co2_ff  , only: declare_emission_co2_ff    , free_emission_co2_ff   , calc_emission_co2_ff
  use emission_co2_ocean, only: declare_emission_co2_ocean  , free_emission_co2_ocean  , calc_emission_co2_ocean
  use emission_co2_bio, only: declare_emission_co2_bio  , free_emission_co2_bio  , calc_emission_co2_bio
  use emission_co2_fires  , only: declare_emission_co2_fires    , free_emission_co2_fires   , calc_emission_co2_fires


  implicit none


  private

  ! public routines:
  public   :: Emis_Init, emission_apply, Emis_Done

  ! --- const --------------------------------------

  character(len=*), parameter  ::  mname = 'emission'



contains
   
  subroutine Emis_Init(status)
    !
    !       purpose
    !       -------
    !       allocate space needed to handle the emissions
    !
    !       interface
    !       ---------
    !       call declare_emission is called from trace0 (start and newmonth)
    !
    !       method
    !       ------
    !       dynamic allocation, depending on dimensions problem
    !       opening of daily input files
    !       reading monthly/yearly emissions
    !       
    !---------------------------------------------------------------------   
    use dims,          only : im,jm,lm,newsrun,nregions
    use chem_param,    only : ntracet
    use budget_global, only : nbud_vg
    implicit none
    ! --- in/out --------------------------------
    
    integer, intent(out)           ::  status
    
    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/Emis_Init'

    ! local
    integer                   :: region, imr, jmr, lmr

#ifdef with_budget
    ! on all PEs to be made available:
    do region=1,nregions
      imr = im(region) ;jmr = jm(region) ; lmr = lm(region)
      allocate(budemi_dat(region)%emi(im(region),jm(region),nbud_vg,ntracet))
      budemi_dat(region)%emi     = 0.0
      sum_emission(region)       = 0.0
    enddo
#endif

    call declare_emission_co2_ff  
    call declare_emission_co2_fires  
    call declare_emission_co2_ocean
    call declare_emission_co2_bio
    call declare_emission_co2_all

    status=0

  end subroutine Emis_Init

! ---------------------------------

  subroutine emission_apply(region)
    use Dims,          only: okdebug
    use ParTools,      only: ntracetloc, tracer_active, myid, root_t
    implicit none
    integer, intent(in)        :: region

    if(okdebug) write(*,*) 'start of emission_apply'
    if(ntracetloc == 0) return      !if no tracer on this processor do notihng

    call calc_emission_co2_ff  
    call calc_emission_co2_fires  
    call calc_emission_co2_ocean
    call calc_emission_co2_bio
    call calc_emission_co2_all

    call emission_apply_co2_all(region)

    if(okdebug) write(*,*) 'End of adding emission ' 

  end subroutine emission_apply

  subroutine Emis_Done(budget_file,status)
    !
    !       purpose
    !       -------
    !       deallocate space needed to handle the emissions
    !
    !       interface
    !       ---------
    !       call free_emission is called from trace_end
    !
    !       method
    !       ------
    !       collecting budgets on coarser budget regions
    !       writing budget file
    !       deallocation of memory
    !       closing of input files
    !---------------------------------------------------------------------   
   use dims,          only : im,jm, nregions
   use budget_global, only : nbudg, nbud_vg, budg_dat, apply_budget_global
   use io_hdf,        only : DFACC_WRITE, io_write
   use chem_param,    only : ntracet
#ifdef MPI
    use mpi_const, only: lmloc, tracer_active, ntracetloc, myid, root, my_real, mpi_sum, com_trac, ierr, root_t
#else
    use ParTools, only: lmloc, tracer_active, myid, root, ntracetloc, root_t
#endif    
   implicit none
   ! --- in/out --------------------------------
    
   integer, intent(out)           ::  status
   character(len=*)               :: budget_file
   
   ! --- const ------------------------------

   character(len=*), parameter ::  rname = mname//'/Emis_Done'

   ! local
   integer                                  :: region, imr, jmr
   integer                                  :: i,j,n, nzone
   real, dimension(nbudg, nbud_vg, ntracet) :: budemig
   real, dimension(:,:,:,:),allocatable     :: budget4d

   integer                                  :: ifail,sfend, nsend, io, sfstart

#ifdef with_budget
   if(apply_budget_global) then

   !handle budget emission:

    if(myid == root) then
       io = sfstart(budget_file, DFACC_WRITE)
    endif

    budemig(:,:,:) = 0.0
    do region = 1, nregions
       allocate(budget4d(im(region),jm(region),nbud_vg,ntracet))
#ifdef MPI
       nsend = im(region)*jm(region)*nbud_vg*ntracet
       call mpi_allreduce(budemi_dat(region)%emi,budget4d, &
            nsend,my_real,mpi_sum,com_trac,ierr)
#else
       budget4d = budemi_dat(region)%emi
#endif
       call io_write(io,im(region),'im',jm(region),'jm', nbud_vg, 'nbud_vg', ntracet,'ntracet', &
              budget4d ,'budemi_dat')
       do n=1,ntracet 
          do j=1,jm(region)
             do i=1,im(region)
                nzone = budg_dat(region)%nzong(i,j)
                budemig(nzone,:,n) = budemig(nzone,:,n) + &
                        budget4d(i,j,:,n) 
             end do
          end do!j
      end do !
      deallocate(budget4d)
   enddo  ! regions

   if(myid == root) then
      call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                   ntracet,'ntracet',budemig,'budemi')
      print *, 'Close budget file:', sfend(io)
   endif

   endif ! apply_budget_global

   do region = 1, nregions
      deallocate(budemi_dat(region)%emi)
   enddo   
#endif
   call free_emission_co2_ff  
   call free_emission_co2_fires  
   call free_emission_co2_ocean
   call free_emission_co2_bio
   call free_emission_co2_all

   status=0

   end subroutine Emis_Done

end module emission
