module emission_co2_ocean
  !-----------------------------------------------------------------------
  !       purpose
  !       -------
  !       perform co2 oceansphere emissions 
  !
  !       interface
  !       ---------
  !       call emission_apply_ocean
  !
  !       method
  !       ------
  !       subroutine is called from its general parent routine apply_emission
  !-----------------------------------------------------------------------
  use Dims,  only        : nregions, nlon360,nlat180
  use Meteo, only        : u10m_sfc,v10m_sfc,ci_sfc
  use Meteo, only        : Set
  implicit none

  private

  public   :: co2_ocean
  ! public routines:
  public   :: declare_emission_co2_ocean, free_emission_co2_ocean, calc_emission_co2_ocean

  real,dimension(nlon360,nlat180),target :: co2_ocean
  character(len=200)              :: deltapco2_prefix


contains

  subroutine declare_emission_co2_ocean
  use GO,            only: TrcFile, Init, Done, ReadRc
  use global_data,   only: rcfile
  implicit none
  integer              :: status
  type(TrcFile)        :: rcF

  call Set(   u10m_sfc, status, used=.true. )
  call Set(   v10m_sfc, status, used=.true. )
  call Set(   ci_sfc, status, used=.true. )

  call Init( rcF, rcfile, status )        
  call ReadRc( rcF, 'deltapco2.prefix', deltapco2_prefix ,status)
  call Done( rcF ,status)

  end subroutine declare_emission_co2_ocean

   subroutine calc_emission_co2_ocean

   use global_data,   only: mass_dat
   use Dims,          only: dx,dy,nlon360, nlat180, im,jm, sec_month, newsrun,  &
                            idate, itau, staggered, nread, newmonth, newday
   use datetime,      only: calc_sm, tstamp, tau2date
   use emission_data, only: chardate, increase_itau
   use io_save,       only: readtm3hdf
   use toolbox,       only: escape_tm
   use ParTools,            only : myid, root ! for debugging

   implicit none
  
   real, parameter                 :: a1=-58.0931
   real, parameter                 :: a2=90.5069
   real, parameter                 :: a3=22.294
   real, parameter                 :: b1=0.027766
   real, parameter                 :: b2=-0.025888
   real, parameter                 :: b3=0.0050578
   integer                         :: i,j,ii,jj,status
   integer, parameter              :: annual_field = 0, rank2 = 2, level1 = 1
   character(len=10)               :: cdate
   integer, dimension(6)           :: idate_ice
   character(len=2),dimension(12)  :: mlab=(/ '01','02','03','04','05','06','07','08','09','10','11','12'/)
   real,dimension(nlon360,nlat180) :: ww,k92,pco2_air,pco2_sw,sol,sc,pbaro,svp,pco2_air_p
   real,pointer,dimension(:,:)     :: u10m,v10m,ci
   real,pointer,dimension(:,:,:)   :: press
   real, parameter, dimension(11)  :: Taka_total = (/    -0.498204, &
                                                          0.139826, &
                                                          0.501043, &
                                                         -0.289709, &
                                                         -0.311620, &
                                                         -0.246089, &
                                                          0.111218, &
                                                         -0.157563, &
                                                         -0.728277, &
                                                          0.160084, &
                                                         -0.472868 /)
   real, parameter, dimension(11)  :: Andy_total = (/    -0.547917, &
                                                          0.073089, &
                                                          0.305705, &
                                                         -0.510675, &
                                                         -0.258474, &
                                                         -0.453971, &
                                                          0.094572, &
                                                         -0.247547, &
                                                         -0.142882, &
                                                          0.072436, &
                                                         -0.528195 /)



! ____ BEGIN ______

  if(mod(itau,nread) /= 0) return  ! only every nread hours

! get necessary fields

   if(newmonth) then

     call readtm3hdf(trim(deltapco2_prefix)//'_pco2_'//mlab(idate(2))//'.hdf',rank2, &
                     nlon360,nlat180,level1,annual_field,pco2_air,'pco2_air')
     call readtm3hdf(trim(deltapco2_prefix)//'_pco2_'//mlab(idate(2))//'.hdf',rank2, &
                     nlon360,nlat180,level1,annual_field,pco2_sw,'pco2_sea')
     call readtm3hdf(trim(deltapco2_prefix)//'_pco2_'//mlab(idate(2))//'.hdf',rank2, &
                     nlon360,nlat180,level1,annual_field,svp,'satvappress')
     call readtm3hdf(trim(deltapco2_prefix)//'_pco2_'//mlab(idate(2))//'.hdf',rank2, &
                     nlon360,nlat180,level1,annual_field,sc,'schmidt')
     call readtm3hdf(trim(deltapco2_prefix)//'_pco2_'//mlab(idate(2))//'.hdf',rank2, &
                     nlon360,nlat180,level1,annual_field,sol,'solubility')
     call readtm3hdf(trim(deltapco2_prefix)//'_pco2_'//mlab(idate(2))//'.hdf',rank2, &
                     nlon360,nlat180,level1,annual_field,pbaro,'pbaro')

   endif

   u10m=> u10m_sfc%data1(:,:,1)
   v10m=> v10m_sfc%data1(:,:,1)
   ci=> ci_sfc%data1(:,:,1)
   press => mass_dat(1)%phlb_t

   pco2_air_p=0.0
   k92=0.0
   co2_ocean=0.0

   ww=u10m**2+v10m**2  ! note: sqrt has been taken away since we square it in the next formula!

! make diffusion in m/s according to Wanninkhof92 param on 1x1 degree

   where(sc(:,:).ge.0) k92=(0.31*(ww)/sqrt(sc/660.0))*1.e-2/3600.   ! only over oceans

! make partial pressure of co2 in air from climatological value corrected for
! pressure in TM5 model, see equation 7 of [Kettle et al, 2004]
   do i=1,nlon360
   do j=1,nlat180
     if(pbaro(i,j).lt.0.0) cycle
     ii=(i-1)/dx+1
     jj=(j-1)/dy+1
     pco2_air_p(i,j)=pco2_air(i,j)*&  ! yields pco2_air_p in uatm
                                            (1.+(pbaro(i,j)-press(ii,jj,1)/100.)/(pbaro(i,j)-svp(i,j)))
   enddo
   enddo

! make fluxes of co2 in mol/m2/s with tm5 meteo and with climatological winds

   co2_ocean=k92*sol*(pco2_sw-pco2_air_p)


   where(pco2_sw(:,:).lt.0.0) co2_ocean(:,:)=0.0  ! reset every value on land to make sure

   co2_ocean=co2_ocean*(1.0-ci) ! multiply out the sea-ice for this day

!   if(myid==root) then
!      print*,'pco2_air statistics mi/ma/sum: ',minval(pco2_air_p),maxval(pco2_air_p),sum(pco2_air_p)
!      print*,'diffusion statistics mi/ma/sum: ',minval(k92),maxval(k92),sum(k92)
!      print*,'Ocean flux statistics mi/ma/sum: ',minval(co2_ocean),maxval(co2_ocean),sum(co2_ocean)
!   endif
!
!   call escape_tm('test')

   nullify(u10m)
   nullify(v10m)
   nullify(ci)
   nullify(press)

  end subroutine calc_emission_co2_ocean

  subroutine free_emission_co2_ocean
  implicit none
  integer               :: region
  end subroutine free_emission_co2_ocean

end module emission_co2_ocean
