!#################################################################
!
! contains calls to user-specific output routines, e.g.
! instantaneous mix files, station output, output of flight tracks etc.
!
!### macro's #####################################################
!
#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!#################################################################

module user_output

  use GO, only : gol, goErr

  implicit none

  ! --- in/out ------------------------

  private

  public :: user_output_init
  public :: user_output_step
  public :: user_output_mean
  public :: user_output_done

  public :: flight_data
  public :: column_data
  public :: column3d_data
  public :: eddyflux_data
  public :: station_data
  public :: mix_data
  public :: mmix_data

  ! --- const -------------------------

  character(len=*), parameter  ::  mname = 'user_output'
  
  ! --- var ---------------------------

  logical           :: flight_data = .false.  ! signal for flight output

  logical           :: column_data = .false.  ! signal for columnoutput
  integer           :: column_data_dhour=24         ! every dhour hour

  logical           :: column3d_data = .false.  ! signal for columnoutput
  integer           :: column3d_data_dhour=24         ! every dhour hour

  logical           :: eddyflux_data = .false.  ! signal for eddyflux output

  logical           :: station_data = .true.  ! signal for station output

  logical           :: mix_data = .false.     ! signal for mix output
  integer           :: mix_data_dhour         ! every dhour hour

  logical           :: mmix_data = .false.     ! signal for mean mix output


contains


  !-------------------------------------------------------------
  ! user_output_init:
  !      Initialise user-specified model output (all regions)
  !-------------------------------------------------------------

  subroutine user_output_init( status )

    use GO,                  only : TrcFile, Init, Done, ReadRc
    use global_data,         only : rcfile
    use user_output_mmix,    only : init_mmix
    use user_output_station, only : read_stationlist, init_station_output, StationlistFFilename 
    use user_output_mix,     only : output_mix_init
    use user_output_column,  only : init_columndata
    use user_output_column3d,only : init_columndata3d
    use user_output_forecast,only : ObslistFFilename
    use user_output_eddyflux, only : read_eddyfluxlist, init_eddyflux_output, eddyfluxlistFFilename 
    
    ! --- in/out -----------------------------
    
    integer, intent(out)      ::  status
    
    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/user_output_init'
    
    ! --- local -------------------------------

    type(TrcFile)       :: rcF

    ! --- begin -------------------------------
    
    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)

    call ReadRc( rcF, 'output.station', station_data, status,default=.false. )
    IF_NOTOK_RETURN(status=1)
    if ( station_data ) then
       call ReadRc( rcF, 'input.station.filename', StationlistFFilename, status)
       IF_NOTOK_RETURN(status=1)
       call read_stationlist
       call init_station_output
    end if

    call ReadRc( rcF, 'output.flight', flight_data, status,default=.false. )
    IF_NOTOK_RETURN(status=1)

    call ReadRc( rcF, 'output.column', column_data, status ,default=.false.)
    IF_NOTOK_RETURN(status=1)
    if ( column_data ) then
      call ReadRc( rcF, 'output.column.dhour', column3d_data_dhour, status , default=24)
      IF_NOTOK_RETURN(status=1)
      call init_columndata
    end if

    call ReadRc( rcF, 'output.column3d', column3d_data, status ,default=.false.)
    IF_NOTOK_RETURN(status=1)
    if ( column3d_data ) then
      call ReadRc( rcF, 'output.column3d.dhour', column3d_data_dhour, status , default=24)
      IF_NOTOK_RETURN(status=1)
      call init_columndata3d
    end if

    call ReadRc( rcF, 'output.eddyflux', eddyflux_data, status,default=.false. )
    IF_NOTOK_RETURN(status=1)
    if ( eddyflux_data ) then
       call ReadRc( rcF, 'input.eddyflux.file',eddyfluxlistFFilename, status)
       IF_NOTOK_RETURN(status=1)
       call read_eddyfluxlist
       call init_eddyflux_output
    end if

    call ReadRc( rcF, 'output.mix', mix_data, status,default=.false. )
    IF_NOTOK_RETURN(status=1)
    if ( mix_data ) then
      call ReadRc( rcF, 'output.mix.dhour', mix_data_dhour, status ,default=24)
      IF_NOTOK_RETURN(status=1)
      call output_mix_init
    end if

    ! initialise accumulation of the mean mixing ratio fields
    call ReadRc( rcF, 'output.mmix', mmix_data, status,default=.false. )
    IF_NOTOK_RETURN(status=1)
    if ( mmix_data ) then
      call init_mmix 
    end if  

    ! close rcfile:
    call Done( rcF, status  )
    IF_NOTOK_RETURN(status=1)

    ! ok
    status = 0

  end subroutine user_output_init


  !-------------------------------------------------------------
  ! user_output_done:
  !      Finalise user-specified model output for the region given
  !-------------------------------------------------------------

  subroutine user_output_done( status )

    use user_output_mmix,     only : write_mmix, free_mmix
    use user_output_station,  only : close_stationfiles, write_stationfiles
    use user_output_eddyflux, only : close_eddyfluxfiles, write_eddyfluxfiles
    use user_output_mix,      only : output_mix_close
    use user_output_column,   only : free_columndata  
    use user_output_column3d,   only : free_columndata3d  
    use dims,                 only : nregions
    
    ! --- in/out -----------------------------
    
    integer, intent(out)      ::  status
    
    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/user_output_done'
    
    ! --- begin -----------------------------
    
    ! write the mean mixing ratio fields to file
    if ( mmix_data ) then
      call write_mmix
      call free_mmix
    end if

    if(column_data ) call free_columndata 
    
    if(column3d_data ) call free_columndata3d 
    
    if ( write_stationfiles .and. station_data) call close_stationfiles

    if ( write_eddyfluxfiles .and. eddyflux_data) call close_eddyfluxfiles

    if ( mix_data ) call output_mix_close

    ! ok
    status = 0

  end subroutine user_output_done

  SUBROUTINE user_output_mean

    use dims,                     only: itau, ndyn_max
    use dims,                     only: nregions
    use dims,                     only: revert
    use dims,                     only: newmonth, newday

    use datetime,                 only: tau2date

    use user_output_column,       only: evaluate_columndata
    use user_output_column,       only: write_columndata
    use user_output_column,       only: reset_columndata_accumulator
    use user_output_column3d,     only: evaluate_columndata3d
    use user_output_column3d,     only: write_columndata3d
    use user_output_column3d,     only: reset_columndata_accumulator3d
    use user_output_station,      only: evaluate_stationconc
    use user_output_station,      only: reset_stationconc_accumulator
    use user_output_station,      only: write_stationconc

    use user_output_eddyflux,      only: evaluate_eddyflux
    use user_output_eddyflux,      only: reset_eddyflux_accumulator
    use user_output_eddyflux,      only: write_eddyflux
 

    use user_output_mmix,         only: reset_mmix
    use user_output_mmix,         only: write_mmix


    IMPLICIT NONE

    !__IO___________________________________________________________________

    !__LOCAL_VARIABLES______________________________________________________

    INTEGER ::  region,statuss
    integer,dimension(6)            :: idate_f

    !__START_SUBROUTINE______________________________________________________

    IF(station_data)THEN
      IF(mod(itau, ndyn_max) == 0) THEN
         CALL evaluate_stationconc
         CALL write_stationconc
         CALL reset_stationconc_accumulator  
      ENDIF
    ENDIF

    IF(eddyflux_data)THEN
      IF(mod(itau, ndyn_max) == 0) THEN
         CALL evaluate_eddyflux
         CALL write_eddyflux
         CALL reset_eddyflux_accumulator  
      ENDIF
    ENDIF

    IF(column_data)THEN
      do region=1,nregions
         call tau2date(itau,idate_f)
         if ( modulo(idate_f(4),column_data_dhour) == 0 ) then
             CALL evaluate_columndata(region)
             CALL write_columndata(region,statuss)
             CALL reset_columndata_accumulator(region)    
         endif
      enddo
    ENDIF

    IF(column3d_data)THEN
      do region=1,nregions
         call tau2date(itau,idate_f)
         if ( modulo(idate_f(4),column3d_data_dhour) == 0 ) then
             CALL evaluate_columndata3d(region)
             CALL write_columndata3d(region,statuss)
             CALL reset_columndata_accumulator3d(region)    
         endif
      enddo
    ENDIF


    !===============
    ! mmix output
    !===============
!cmk: this creates double writing of mmix at end of monthly run.....removed!
!cmk: double writing means loss of data.....
!cmk     IF(mmix_data)THEN
!cmk 
!cmk         IF(newmonth) THEN
!cmk !        IF(newday) THEN
!cmk           CALL write_mmix
!cmk           CALL reset_mmix
!cmk         ENDIF
!cmk        
!cmk     ENDIF



  END SUBROUTINE  

!===========================================================================================================


  !-------------------------------------------------------------
  ! user_output_step:
  !      Define user-specified model output for the region given
  !      Called every time step
  !-------------------------------------------------------------

  subroutine user_output_step( region, status )

    use dims,                 only : itaur
    use datetime,             only : tau2date
    use user_output_station,  only : output_stationconc  
    use user_output_eddyflux,  only : output_eddyflux  
    use user_output_column,   only : output_columndata  
    use user_output_column3d,   only : output_columndata3d  
    use user_output_flight,   only : get_flightdata
    use user_output_mix,      only : output_mix
    use user_output_mmix,     only : accumulate_mmix
    use user_output_forecast, only : get_forecast

    ! --- in/out ------------------------------
    
    integer, intent(in)       ::  region
    integer, intent(out)      ::  status
    
    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/user_output_step'
    
    ! --- local ------------------------------
    
    integer,dimension(6)            :: idate_f

    ! --- begin ------------------------------

    call tau2date(itaur(region),idate_f)

    if ( mmix_data ) call accumulate_mmix( region )

    if ( station_data ) call output_stationconc(region)

    if ( eddyflux_data ) call output_eddyflux(region)

    if ( column_data ) call output_columndata(region)

    if ( column3d_data ) call output_columndata3d(region)

    if ( flight_data )  call get_flightdata(region,idate_f)

    call get_forecast(region,itaur(region),.false.)

    if ( mix_data ) then
      if ( modulo(idate_f(4),mix_data_dhour) == 0 ) call output_mix(region)
    end if

    ! ok
    status = 0

  end subroutine user_output_step


end module user_output
