module user_output_forecast

  use dims,       only : im, jm, lm, dx, xref, dy, yref, xbeg, xend, ybeg, yend
  use dims,       only : nregions, region_name, itaui,itaue, xcyc 
  use chem_param, only : fscale
  use ParTools,   only : myid, root, maxtrac_per_pe, ntracetloc, par_barrier

  implicit none


  character(len=100)           :: ObslistFFilename = ''
  integer                      :: n_fcast
  integer,dimension(6)         :: idate_fcast
  integer                      :: itau_fcast
  integer,parameter            :: window = 4*3600 ! 2 hrs around obs hour
  type sites
    integer,dimension(6)       :: idate     ! date of observation
    character(len=7)           :: ident     ! station identifier
    real                       :: lat       ! latitude of station
    real                       :: lon       ! longitude of station
    real                       :: height    ! height of station
    character(len=30)          :: mbl       ! mbl site or not

    real                            :: obs=0.0      ! observed value (from file)
    real,dimension(maxtrac_per_pe)  :: model=0.0    ! model predicted background value
    real,dimension(maxtrac_per_pe)  :: residual=0.0 ! obs-(bck+hx_prime)
    real                            :: flag=0.0
    real                            :: hqhr=0.0     ! value of HQH+R for innovation statistics
    real                            :: error=0.0    ! model data mismatch
    integer                         :: code=0

    integer                    :: itau=0
    integer                    :: n_accumulate=0  ! nr of times concentration was added in get_forecast       

    integer                    :: region 
    integer                    :: is
    integer                    :: js
    integer                    :: lsmin
    integer                    :: lsmax
  end type sites

  type(sites), dimension(:), allocatable  :: concentration

  private

  ! private 

  integer,parameter           :: nf_trace =  1
  real                        :: rmf
  ! number of locations to be calculated for 1 model time
  integer,dimension(nf_trace) :: if_trace = (/ 1 /)

  public :: read_co2_obs, get_forecast, concentration, sites, n_fcast, itau_fcast, idate_fcast, ObslistFfilename
  public :: write_co2_pseudo

contains

   subroutine read_co2_obs
   use Meteo,      only : Set
   use Meteo,      only : temper_dat, humid_dat, gph_dat
   use binas,      only : ae, twopi, grav
   use dims,       only : im, jm, lm, dx, dy, xref, yref, xbeg, ybeg, xend, yend
   use dims,       only : nregions, region_name, itaur, xcyc
   use chem_param, only : ntrace, ntracet, names, fscale
   use toolbox,    only : escape_tm
   use datetime,   only : date2tau
   use go_string,  only : goLoCase   
#ifdef MPI
   use mpi_const,  only : my_real,mpi_comm_world,ierr,mpi_integer,mpi_character
#endif
   implicit none

   ! local
   integer                           :: slen1 !string length
   integer                           :: slen2 !string length
   integer                           :: slen3 !string length
   character(len=200)                :: dummy
   integer                           :: fstatus 
   integer                           :: i_stat
   integer                           :: sunit0=333
   integer                           :: region
   integer,dimension(6)              :: idate_sim
   integer                           :: itau_sim, isim, ispec
     ! x,y resolution (in degrees) for current region 
   real                              :: dxr, dyr 
   character(len=8)                  :: name_spec    !CMK changed from 6-->8
   integer           ::  status

     ! start

     call Set(    gph_dat, status, used=.true. )
     call Set(    temper_dat, status, used=.true. )
     call Set(    humid_dat, status, used=.true. )

     if(myid==root) then

        !=========================================
        ! open StationFile (with list of stations)
        !=========================================

        open(UNIT=sunit0, FORM='FORMATTED', &
             FILE='./'//trim(ObslistFFilename) , &
             ERR=1000)      

        ! count number of stations
        n_fcast=0
        stat_loop: DO
           read(sunit0,  '(a)', END=100) dummy
           n_fcast = n_fcast+1 
        end do stat_loop
100       continue
        print *, '_________________________________________________' &
             //'_______________________________'
        print *, n_fcast, ' stations read from file: ', ObslistFFilename
        print *, '_________________________________________________' &
             //'_______________________________'

     endif

#ifdef MPI    
     call MPI_BCAST(n_fcast           ,1,      MPI_INTEGER, root ,MPI_COMM_WORLD,ierr)  
#endif

     if(n_fcast.eq.0) call escape_tm( 'read_co2_obs: no obs in file ' // ObslistFFilename) 

     if(allocated(concentration)) deallocate(concentration)  ! avoid double allocation

     allocate(concentration(n_fcast))


     dyr = dy/yref(1)
     dxr = dx/xref(1)

     !===============
     ! read stations
     !===============

     if(myid==root) then
        rewind(sunit0)
        do i_stat=1, n_fcast 

        read(sunit0,  '(a7,1x,f8.2,f8.2,f8.1,1x,6i4,x,f6.2,x,a3,x,a30)') &
           concentration(i_stat)%ident, &
           concentration(i_stat)%lat, &
           concentration(i_stat)%lon, &
           concentration(i_stat)%height, &
           concentration(i_stat)%idate, &
           concentration(i_stat)%obs, &
           concentration(i_stat)%flag, &
           concentration(i_stat)%mbl

        print('(a7,1x,f8.2,f8.2,f8.1,1x,6i4,x,f6.2,x,a3,x,a30)'), &
           concentration(i_stat)%ident, &
           concentration(i_stat)%lat, &
           concentration(i_stat)%lon, &
           concentration(i_stat)%height, &
           concentration(i_stat)%idate, &
           concentration(i_stat)%obs, &
           concentration(i_stat)%flag, &
           concentration(i_stat)%mbl

           concentration(i_stat)%ident=goLoCase(trim(adjustl(concentration(i_stat)%ident)))
 
           call date2tau(concentration(i_stat)%idate,concentration(i_stat)%itau)  ! fill itau

           if(abs(concentration(i_stat)%itau-itaui).lt.1) concentration(i_stat)%itau=concentration(i_stat)%itau+3600
           if(abs(concentration(i_stat)%itau-itaue).lt.1) concentration(i_stat)%itau=concentration(i_stat)%itau-3600

           concentration(i_stat)%is = &
                int((concentration(i_stat)%lon-float(xbeg(1)))/dxr + 0.99999) 
           concentration(i_stat)%js = &
                int((concentration(i_stat)%lat-float(ybeg(1)))/dyr + 0.99999)
           concentration(i_stat)%lsmin = 999
           concentration(i_stat)%lsmax = 0
        end do

        ! close stationfile
        close(sunit0)

        ! move some sites by one degree on/off coast

        where((concentration%ident) == 'brw    ') concentration%lat=concentration%lat+1.0
        where((concentration%ident) == 'brwi   ') concentration%lat=concentration%lat+1.0
        where((concentration%ident) == 'bhd    ') concentration%lat=concentration%lat-1.0
        where((concentration%ident) == 'bhd    ') concentration%lon=concentration%lon+1.0
        where((concentration%ident) == 'bsc    ') concentration%lon=concentration%lon+1.0
        where((concentration%ident) == 'cba    ') concentration%lat=concentration%lat-1.0
        where((concentration%ident) == 'cgo    ') concentration%lon=concentration%lon-1.0
        where((concentration%ident) == 'cmo    ') concentration%lon=concentration%lon-1.0
        where((concentration%ident) == 'cpt    ') concentration%lat=concentration%lat-1.0
        where((concentration%ident) == 'esp    ') concentration%lon=concentration%lon-1.0
        where((concentration%ident) == 'ice    ') concentration%lat=concentration%lat-1.0
        where((concentration%ident) == 'key    ') concentration%lon=concentration%lon+1.0
        where((concentration%ident) == 'uta    ') concentration%lon=concentration%lon-1.0
        where((concentration%ident) == 'mhd    ') concentration%lon=concentration%lon-1.0
        where((concentration%ident) == 'tap    ') concentration%lat=concentration%lat-1.0
        where((concentration%ident) == 'tap    ') concentration%lon=concentration%lon-1.0
        where((concentration%ident) == 'pta    ') concentration%lon=concentration%lon-1.0
        where((concentration%ident) == 'thd    ') concentration%lon=concentration%lon-1.0

     endif

        ! determine assignment of station to region 
        ! (with highest refinement factor)

#ifdef MPI    
     do i_stat=1, n_fcast 
         call MPI_BCAST(concentration(i_stat)%ident ,7,        MPI_CHARACTER, &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%lat   ,1,        MY_REAL      , &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%lon   ,1,        MY_REAL      , &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%height,1,        MY_REAL      , &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%idate,6,        MPI_INTEGER      , &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%mbl,30,        MPI_CHARACTER      , &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%flag,1,        MY_REAL      , &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%itau,1,        MPI_INTEGER      , &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%obs,1,        MY_REAL,   &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%is    ,1,        MPI_INTEGER,   &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%js    ,1,        MPI_INTEGER,   &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%lsmin ,1,        MPI_INTEGER,   &
              root ,MPI_COMM_WORLD,ierr)  
         call MPI_BCAST(concentration(i_stat)%lsmax ,1,        MPI_INTEGER,   &
              root ,MPI_COMM_WORLD,ierr)  
     end do
#endif
     do i_stat=1, n_fcast

        ! assume global region as default for average mixing ratios
        concentration(i_stat)%region = 1
        concentration(i_stat)%is = 1
        concentration(i_stat)%js = 1
        concentration(i_stat)%n_accumulate = 0 ! no model values addded yet
        concentration(i_stat)%model = 0.0 ! no model values addded yet

        do region=1, nregions

           dyr = dy/yref(region)
           dxr = dx/xref(region)

           if ( (concentration(i_stat)%lon .gt. xbeg(region) .and. &
                 concentration(i_stat)%lon .lt. xend(region)) .and.      &
                (concentration(i_stat)%lat .gt. ybeg(region) .and. &
                 concentration(i_stat)%lat.lt.yend(region) ) ) then
              !=====================
              ! station is in region
              !=====================

              ! determine region with hightes refinement factor
              !if (xref(region) > xref(concentration(i_stat)%region)) then
                 concentration(i_stat)%region = region
                 concentration(i_stat)%is = &
                   int((concentration(i_stat)%lon-float(xbeg(region)))/dxr + 0.99999) 
                 concentration(i_stat)%js = &
                   int((concentration(i_stat)%lat-float(ybeg(region)))/dyr + 0.99999)
              !end if
           end if
        end do
     end do

     call Par_Barrier

     if(myid == root) then
        do i_stat=1, n_fcast 
           print '(a7,1x,f8.2,f8.2,f8.1,1x,3i4,1x,f6.2)', &
                concentration(i_stat)%ident, &
                concentration(i_stat)%lat, &
                concentration(i_stat)%lon, &
                concentration(i_stat)%height, &
                concentration(i_stat)%region, &
                concentration(i_stat)%is, &
                concentration(i_stat)%js, &
                concentration(i_stat)%obs
        end do
     end if


     return

     ! error handling
 1000 call escape_tm( 'read_co2_obs: could not open ' // ObslistFFilename) 

   end subroutine read_co2_obs

   subroutine write_co2_pseudo
   use binas,      only : ae, twopi, grav
   use dims,       only : im, jm, lm, dx, dy, xref, yref, xbeg, ybeg, xend, yend
   use dims,       only : nregions, region_name, itaur, xcyc
   use chem_param, only : ntrace, ntracet, names, fscale
   use toolbox,    only : escape_tm
   use datetime,   only : date2tau

   implicit none

   ! local
   integer                           :: slen1 !string length
   integer                           :: slen2 !string length
   integer                           :: slen3 !string length
   character(len=200)                :: dummy
   integer                           :: fstatus 
   integer                           :: i_stat
   integer                           :: sunit0=333
   real                              :: pseudo_obs

     ! start

     
     if ( myid == root ) then

        !WP! use outputarray a we don't want this routine to change the
        !concentrations which we still need for the HDF output


        !=========================================
        ! open StationFile (with list of stations)
        !=========================================

        open(UNIT=sunit0, FORM='FORMATTED', &
             FILE='./pseudo_co2.list' , &
             ERR=1000,status='replace')      

        do i_stat=1, n_fcast 

        !WP! use pseudo_obs a we don't want this routine to change the
        !concentrations which we still need for the HDF output
        pseudo_obs=concentration(i_stat)%model(1)*1.e6
        if(concentration(i_stat)%n_accumulate.ge.1)  &
                pseudo_obs=pseudo_obs/concentration(i_stat)%n_accumulate

        write(sunit0,  '(a7,1x,f8.2,f8.2,f8.1,1x,6i4,x,f6.2,x,a3,x,a30)') &
           concentration(i_stat)%ident, &
           concentration(i_stat)%lat, &
           concentration(i_stat)%lon, &
           concentration(i_stat)%height, &
           concentration(i_stat)%idate, &
           pseudo_obs, &
           concentration(i_stat)%flag, &
           concentration(i_stat)%mbl

        end do

        ! close stationfile
        close(sunit0)

      end if ! root

     return

     ! error handling
 1000 call escape_tm( 'write_co2_pseudo: could not open pseudo_co2.list') 

   end subroutine write_co2_pseudo

    subroutine get_forecast(region,itau_f,force)
    !
    ! This subroutine samples the model at given locations in array
    ! concentrations and puts the values in the tag: forecast
    ! This is done only in a certain time window, or when the keyword /force is
    ! used. Thus, a model field can be sampled at sites instantaneously (force),
    ! or at appropriate time ( omit force)
    !
    ! Note: the values accumulate so care should be taken to clear the forecast
    ! and n_accumulate tags between functions
    !
    !
    use global_data, only: mass_dat, region_dat 
    use Meteo,       only: gph_dat
    use ParTools,    only: myid,ntracet_ar

    implicit none

    ! input/output
    integer,intent(in)              :: region
    ! idate_f: date for which output required...
    integer,intent(in)              :: itau_f
    logical,optional                :: force

    ! local
    real,dimension(:,:,:), pointer      :: m,gph
    real,dimension(:,:,:,:), pointer    :: rm, rxm, rym, rzm

    real,dimension(0:lm(region)) :: height
    integer   :: i,is,js,l,n,isn,jsn,ls,j,offsetj, lst, lstn, lmr, lsn
    real      :: flon,flat,falt,ris,rjs,dxr,dyr,wcx,wcy,rls, wcz

    ! start

    m => mass_dat(region)%m_t    !pointers to global arrays...
    rm => mass_dat(region)%rm_t
    rxm => mass_dat(region)%rxm_t
    rym => mass_dat(region)%rym_t
    rzm => mass_dat(region)%rzm_t
    gph => gph_dat(region)%data

    lmr=lm(region)

    fcast_loop: do n=1,n_fcast 

    !  0. Is idate equal to idate_site
    !  1. Is the site in the area?--->no, then put -1 in c
    !  2. Determine gridbox
    !  3. Use slopes to determine concentration at the site.

    if ( (concentration(n)%itau.ge.itau_f-window/2.and.&
          concentration(n)%itau.lt.itau_f+window/2).or.force ) then


       dyr = dy/yref(region)
       dxr = dx/xref(region)

       flon=concentration(n)%lon
       flat=concentration(n)%lat
       falt=concentration(n)%height

       IF (concentration(n)%region == region)THEN
          ris = (flon-float(xbeg(region)))/dxr + 0.99999
          rjs = (flat-float(ybeg(region)))/dyr + 0.99999
          is  = int(ris)   ! i-index of grid cell in which station is located
          js  = int(rjs)   ! j-index of grid cell in which station is located
          !fraction from the center of the is-box  (-0.5---+0.5)
          ris = ris-is-0.5
          !idem js
          rjs = rjs-js-0.5

          !=================================
          !the neighbour for x interpolation
          !=================================
          if ( ris .gt. 0 ) then 
             isn = is+1     
          else
             isn = is-1
          endif

          !=================================
          !the neighbour for y interpolation
          !=================================
          if ( rjs .gt. 0 ) then 
             jsn = js+1     
          else
             jsn = js-1
          endif

          ! x- / y-weighting of grid cell in which station is located
          wcx = (1.0-abs(ris))    ! 1.0 ... 0.5
          wcy = (1.0-abs(rjs))    ! 1.0 ... 0.5

          !=================================================================
          ! if index of neighbour is exceeding range of region set 
          ! neighbour = current cell (i.e. no interpolation)
          ! in case of cyclic x-boundaries take corresponding cyclic i index
          !=================================================================
          if ( jsn < 1) jsn=1
          if ( jsn > jm(region) ) jsn=jm(region)  ! isn-->jsn (wouter Peters)
          if ( xcyc(region) == 0 ) then
             ! non-cyclic boundaries
             if ( isn < 1) isn=1
             if ( isn > im(region) ) isn=im(region)
          else
             ! cyclic x-boundaries
             if ( isn < 1 ) isn=im(region)
             if ( isn > im(region) ) isn=1
          end if

          ! interpolate the altitude to site position...
          ls = 1   !layer
          do l=0,lm(region)
             height(l) =     wcx *      wcy*  gph(is,js,l+1)  + &
                  (1.0-wcx)*      wcy*  gph(isn,js,l+1) + &
                  wcx *(1.0-wcy)*  gph(is,jsn,l+1) + &
                  (1.0-wcx)*(1.0-wcy)*  gph(isn,jsn,l+1)
          enddo
          do l=2,lm(region)   ! selects layer , note that we start from second layer from surface
             if(height(l).gt.falt) exit
          enddo

          select case(l)
          case(0)
             if(myid==root) print*,'get_forecast: Warning..., forecast altitude ',&
                  'is below the surface height',height(0),' > ',falt,concentration(n)%ident
             ls = 1
             rls = -0.5   !surface...
          case default
             ls = l       !the site layer
             ! the off-set from the center of the layer (-0.5--->+0.5)
             ! (interpolation is in (m))
             rls = (falt-height(l-1))/(height(l)-height(l-1)) - 0.5
          end select
          !=================================
          !the neighbour for z interpolation
          !=================================
          if ( rls .gt. 0 ) then 
             lsn = ls+1     
          else
             lsn = ls-1
          end if
          ! z-weighting of grid cell in which station is located 
          wcz = (1.0-abs(rls))  !.0 ... 0.5

          !=========================================================
          ! if vertical neighbor is 0 (which does not exist)
          ! take vertical layer with l=2 for EXTRApolation to ground
          !=========================================================

          IF(lsn == 0) THEN
            lsn=2
            wcz=1.0-rls  ! 1.0 ... 1.5
          ENDIF    
          IF(lsn == lmr+1) THEN
             !=========================================================
             ! if vertical neighbor is lmr+1 (which does not exist)
             ! -> no interpolation
             !=========================================================
             lsn=lmr      ! no interpolation 
             wcz=1.0
          ENDIF    


          !from is,js,ls, ris,rjs,rls, determine the mixing ratio of each tracer  ...
          do j=1,ntracetloc
             ! rm-value is obtained from rm + slopes. 
             ! slope = rxm = (rm*dX/dx *deltaX/2)

             offsetj = sum(ntracet_ar(0:myid-1))

!             if (lstn <= lmr) then   
!                   rmf = wcx      *wcy      *wcz         *rm(is,js,ls,j)   /m(is,js,ls)   + & 
!                        (1.0-wcx)*wcy      *wcz         *rm(isn,js,ls,j)  /m(isn,js,ls)  + &
!                        wcx      *(1.0-wcy)*wcz         *rm(is,jsn,ls,j)  /m(is,jsn,ls)  + &
!                        (1.0-wcx)*(1.0-wcy)*wcz         *rm(isn,jsn,ls,j) /m(isn,jsn,lst) + &
!                        wcx      *wcy      *(1.0-wcz)   *rm(is,js,lstn,j)  /m(is,js,lstn)  + & 
!                        (1.0-wcx)*wcy      *(1.0-wcz)   *rm(isn,js,lstn,j) /m(isn,js,lstn) + &
!                        wcx      *(1.0-wcy)*(1.0-wcz)   *rm(is,jsn,lstn,j) /m(is,jsn,lstn) + &
!                        (1.0-wcx)*(1.0-wcy)*(1.0-wcz)   *rm(isn,jsn,lstn,j)/m(isn,jsn,lstn) 
!             else  
!PB                   do NOT apply factor wcz here (no interpolation in z direction in this case) !!!!!

!                   rmf = wcx      *wcy          *rm(is,js,ls,n)   /m(is,js,ls)   + & 
!                                 (1.0-wcx)*wcy          *rm(isn,js,ls,n)  /m(isn,js,ls)  + &
!                                 wcx      *(1.0-wcy)    *rm(is,jsn,ls,n)  /m(is,jsn,ls)  + &
!                                 (1.0-wcx)*(1.0-wcy)    *rm(isn,jsn,ls,n) /m(isn,jsn,ls) 
!              endif

             rmf = (      rm(is,js,ls,j) + &
                2.0*(ris*rxm(is,js,ls,j) + &
                     rjs*rym(is,js,ls,j) + &
                     rls*rzm(is,js,ls,j) ) )/m(is,js,ls)*fscale(offsetj+j) 

!             rmf = rm(is,js,ls,j)/m(is,js,ls)*fscale(offsetj+j) 
             concentration(n)%model(j)=concentration(n)%model(j)+rmf !*fscale(offsetj+j)

          enddo

          concentration(n)%n_accumulate=concentration(n)%n_accumulate+1
          !print*,'Check ',myid, concentration(n)%ident,concentration(n)%n_accumulate,concentration(n)%model(:)/concentration(n)%n_accumulate

       endif
    endif ! force/time

    enddo fcast_loop

    nullify(m)
    nullify(rm)
    nullify(rxm)
    nullify(rym)
    nullify(rzm)
    nullify(gph)


  end subroutine get_forecast


end module user_output_forecast
