module user_output_mix

  ! module included to write instantaneous output fields
  ! called from user_output, and controlg.f90 (on exit, to close files).

  use dims,  only : nregions

  implicit none

  ! interface

  private

  public :: output_mix
  public :: output_mix_init, output_mix_close

  ! local

  ! buffer to store multiple fields to prevent slow down.....
  integer, parameter   :: maxbuf =  30
  ! number of output steps that is skipped
  integer, parameter   :: skipbuf = 0
  ! the number that is set on start...
  integer, parameter   :: initbuf = 0
  ! the tracer for which instantaneous output is required...
  integer, parameter   :: itracer_out =   1   ! ....


  type mixfile
     real,dimension(:,:,:),pointer :: field  ! the actual field (allocated!)
     real,dimension(:,:,:),pointer :: pso    ! the correspoding surface pressures!
     character(len=100) :: fname
     integer  :: funit                       ! unit number 
     real     :: lon_first                   ! x geometry field
     real     :: lon_inc                     ! x geometry field
     real     :: lat_first                   ! y geometry field
     real     :: lat_inc                     ! y geometry field
     integer  :: lon_n                       ! x dimension requested field
     integer  :: lat_n                       ! y dimension requested field
     integer  :: ibuf 
     integer  :: skipper 
     integer,dimension(6,maxbuf)   :: idate 
  end type mixfile

  type(mixfile),dimension(nregions)  :: mixf  

  logical :: okdebug = .true.


contains


  subroutine output_mix_init
    !
    !
    use chem_param,  only : ntracet
    use toolbox ,    only : escape_tm
    use dims,        only : nregions
    use dims,        only : im, jm, lm, region_name, parent,  itaur
    use dims,        only : xbeg, xend, ybeg, yend, dx, dy, xref, yref
    use dims,        only : zbeg, zend, dz, zref
    use io_hdf,      only : io_write2d_32d, DFACC_CREATE, DFACC_WRITE
    use io_hdf,      only : DFNT_FLOAT64, DFNT_CHAR, DFNT_INT32
    use global_data, only : region_dat
    use global_data, only : outdir
    use ParTools   , only : myid, root_k, root_t

    implicit none

    ! local
    integer               :: region
    integer               :: io,istat,i,j
    integer               :: sfstart, sfscatt, sfsnatt, sfend

    ! start

    if ( itracer_out > ntracet ) &
         call escape_tm('output_mix_init: Requested tracer > ntracet ' )

    if ( root_t /= root_k )  &
         call escape_tm('output_mix_init: root_t /= root_k MPI ')

    do region = 1, nregions

       if ( myid == root_t ) then 

          !  initialize the mix output:
          allocate(mixf(region)%field(im(region),jm(region),maxbuf))
          allocate(mixf(region)%pso(im(region),jm(region),maxbuf))
          mixf(region)%ibuf  = 0
          mixf(region)%fname = 'mix_'//region_name(region)//'.hdf'
          mixf(region)%funit = sfstart(trim(outdir)//'/'//mixf(region)%fname, DFACC_CREATE)
          mixf(region)%skipper = initbuf
          mixf(region)%lon_first = xbeg(region) + 0.5*dx/xref(region)
          mixf(region)%lat_first = ybeg(region) + 0.5*dy/yref(region)
          mixf(region)%lat_inc   = dy/yref(region)
          mixf(region)%lon_inc   = dx/xref(region)
          mixf(region)%lon_n     = im(region)
          mixf(region)%lat_n     = jm(region)
          io = mixf(region)%funit
          if(okdebug) print *, 'output_mix_init: Mix file opened for region:',&
               region
          istat = sfscatt(io,'region_name', &
               DFNT_CHAR, 6, region_name(region))
          istat = sfsnatt(io,'im',    DFNT_INT32, 1, im(region))
          istat = sfsnatt(io,'jm',    DFNT_INT32, 1, jm(region))
          istat = sfsnatt(io,'lm',    DFNT_INT32, 1, lm(region))
          istat = sfsnatt(io,'dx',    DFNT_FLOAT64, 1, dx/xref(region))
          istat = sfsnatt(io,'dy',    DFNT_FLOAT64, 1, dy/yref(region))
          istat = sfsnatt(io,'dz',    DFNT_FLOAT64, 1, dz/zref(region))
          istat = sfsnatt(io,'xbeg',  DFNT_INT32, 1, xbeg(region))
          istat = sfsnatt(io,'xend',  DFNT_INT32, 1, xend(region))
          istat = sfsnatt(io,'ybeg',  DFNT_INT32, 1, ybeg(region))
          istat = sfsnatt(io,'yend',  DFNT_INT32, 1, yend(region))
          istat = sfsnatt(io,'zbeg',  DFNT_INT32, 1, zbeg(region))
          istat = sfsnatt(io,'zend',  DFNT_INT32, 1, zend(region))
          istat = sfsnatt(io,'lon_first', DFNT_FLOAT64,  &
               1, mixf(region)%lon_first)
          istat = sfsnatt(io,'lat_first', DFNT_FLOAT64,  &
               1, mixf(region)%lat_first)
          istat = sfsnatt(io,'lon_inc'  , DFNT_FLOAT64,  &
               1, mixf(region)%lon_inc)
          istat = sfsnatt(io,'lat_inc'  , DFNT_FLOAT64,  &
               1, mixf(region)%lat_inc)
          istat = sfsnatt(io,'xref '    , DFNT_INT32  ,  &
               1, xref(region))
          istat = sfsnatt(io,'yref '    , DFNT_INT32, &
               1, yref(region))
          istat = sfsnatt(io,'xrefp'    , DFNT_INT32, &
               1, xref(region)/xref(parent(region)))
          istat = sfsnatt(io,'yrefp'    , DFNT_INT32, &
               1, yref(region)/yref(parent(region)))
          istat = sfsnatt(io,'dxyp'     , DFNT_FLOAT64, &
               jm(region) , region_dat(region)%dxyp)
          i = sfend(mixf(region)%funit)   ! close file.....
          mixf(region)%funit = -1         ! reset number 

       else    ! other PEs keep track of buffer
          mixf(region)%ibuf  = 0
          mixf(region)%skipper = initbuf
       end if !root_t

    end do  ! nregions

  end  subroutine output_mix_init



  subroutine output_mix_close
    !-----------------------------------------------------------
    !
    !-----------------------------------------------------------
    use dims,        only : nregions, im,jm 
    use chem_param,  only : names
    use io_hdf,      only : io_write2d_32d, DFACC_WRITE
    use ParTools   , only : myid, root_t

    ! local
    integer               :: region
    integer               :: istat,i,j
    integer               :: sfstart, sfend

    ! start

    if(myid == root_t) then

       do region = 1, nregions
          if ( mixf(region)%funit < 0 ) mixf(region)%funit = &
               sfstart(mixf(region)%fname, DFACC_WRITE)
          if ( okdebug ) print *, mixf(region)%fname
          do i=1,mixf(region)%ibuf 
             if ( okdebug ) print *, 'output_mix_close: ', &
                  'Writing mix file output; region, i = ',region,i
             call io_write2d_32d(mixf(region)%funit,im(region),'LON', &
                  jm(region), 'LAT', mixf(region)%field(:,:,i), &
                  names(itracer_out), &
                  mixf(region)%idate(:,i) )
             call io_write2d_32d(mixf(region)%funit,im(region),'LON', &
                  jm(region), 'LAT', mixf(region)%pso(:,:,i), &
                  'PS', &
                  mixf(region)%idate(:,i) )
          end do
          i = sfend(mixf(region)%funit)   ! close file.....
          if ( okdebug ) print *, 'output_mix_close: ', &
               'Mix file close for region:', region, ' status = ',i
          deallocate(mixf(region)%field)   ! free memory
          deallocate(mixf(region)%pso)   ! free memory
       end do

    endif

  end subroutine output_mix_close



  subroutine output_mix(region)
    !-----------------------------------------------------------
    !
    !-----------------------------------------------------------
    use global_data, only : mass_dat, region_dat
    use meteo      , only : sp_dat
    use dims,        only : im,jm,lm, itaur
    use chem_param,  only : names, ra, fscale, ntracet
    use binas    ,   only : Avog,Dobs
    use io_hdf,      only : io_write2d_32d, DFACC_CREATE, DFACC_WRITE, &
         DFNT_FLOAT64, DFNT_CHAR, DFNT_INT32
    use datetime,    only : tau2date
    use toolbox ,    only : escape_tm
    use ParTools   , only : myid, root_k, root_t
#ifdef MPI
    use mpi_const,   only : which_par
    use mpi_comm,    only : gather_tracer_k_3d, gather_tracer_k, gather_tracer_t
#endif   

    implicit none

    ! in/out
    integer, intent(in) :: region

    ! local
    integer               :: io,istat,i,j, imr, jmr, lmr
    integer               :: sfstart, sfscatt, sfsnatt, sfend
    integer, dimension(6) :: idater
    ! from kg/m2 ---> molecules/cm2 ---> Du (first tracer)
    real,parameter        :: fac=1e-4*1e3/ra(itracer_out)*Avog !/Dobs

    ! MPI arrays to gather fields.
    real, dimension(:,:,:,:), allocatable, target      :: rmg
    real, dimension(:,:,:  ), allocatable, target      :: mg
    real,dimension(:,:,:,:),               pointer     :: rm
    real,dimension(:,:,:),                 pointer     :: p
    real,dimension(:,:,:),                 pointer     :: m
    real,dimension(:),                     pointer     :: dxyp

    ! start

    if ( itracer_out > ntracet ) &
         call escape_tm('output_mix: Requested tracer > ntracet in mix ' )

    if( root_t /= root_k )  &
         call escape_tm('output_mix: root_t /= root_k MPI in mix ')

    if ( mixf(region)%ibuf == maxbuf) then    ! write buffer to file..
       mixf(region)%ibuf  = 0

       if ( myid == root_t ) then             ! only this PE does output...

          if ( mixf(region)%funit < 0 ) &
               mixf(region)%funit = sfstart(mixf(region)%fname, DFACC_WRITE)
          if ( okdebug ) print *, mixf(region)%fname
          do i=1,maxbuf
             if ( okdebug ) print *, 'output_mix: Writing Mix file output',&
                  region,i
             call io_write2d_32d(mixf(region)%funit,im(region),'LON',&
                  jm(region), 'LAT', mixf(region)%field(:,:,i), &
                  names(itracer_out), &
                  mixf(region)%idate(:,i) )
             call io_write2d_32d(mixf(region)%funit,im(region),'LON', &
                  jm(region), 'LAT', mixf(region)%pso(:,:,i), &
                  'PS', &
                  mixf(region)%idate(:,i) )
          end do
          i = sfend(mixf(region)%funit)   ! close file.....
          if ( okdebug ) print *, 'output_mix: Mix file close for region:', &
               region, ' status = ',i
          mixf(region)%funit = -1        

       endif

    endif

    ! either skip or calculate output field

    if ( mixf(region)%skipper /= skipbuf ) then
       mixf(region)%skipper = mixf(region)%skipper + 1   ! increment
    else   

#ifdef MPI
       ! point to global arrays. If needed: collect tracer.....
       imr = im(region) ; jmr = jm(region) ; lmr = lm(region)
       allocate(rmg(-1:imr+2,-1:jmr+2,lmr, ntracet))
       allocate(mg(-1:imr+2,-1:jmr+2,lmr))
       if ( which_par == 'tracer' ) then 
          m => mass_dat(region)%m_t
          rm => mass_dat(region)%rm_t
          call gather_tracer_t(rmg,imr,jmr,lmr,2,2,0,ntracet,rm,.false.)
          nullify(rm)   ! reassign pointers
          rm => rmg
       else
          m => mass_dat(region)%m_k
          rm => mass_dat(region)%rm_k
          call gather_tracer_k(rmg,imr,jmr,lmr,2,2,0,ntracet,rm,.false.)
          call gather_tracer_k_3d( mg, imr,jmr,lmr, 2,2,0, m, .false. )
          nullify(m)   ! reassign pointers
          nullify(rm) 
          m => mg
          rm => rmg
       end if
#else
       m => mass_dat(region)%m_t
       rm => mass_dat(region)%rm_t
#endif
       p => sp_dat(region)%data
       dxyp=> region_dat(region)%dxyp

       mixf(region)%skipper = 0   ! reset skipper
       mixf(region)%ibuf = mixf(region)%ibuf + 1

       if ( myid == root_t) then   !should be eq. to root_k

          call tau2date(itaur(region),idater)
          mixf(region)%idate(:, mixf(region)%ibuf) = idater ! store time
          do j=1,jm(region)
             do i=1,im(region)
                !mixf(region)%field(i,j,mixf(region)%ibuf) = 
                !     fac*sum(rm(i,j,:,1))/dxyp(j)*1e5/p(i,j)
                ! total CO columns  (--->Du in stp)
                ! mixf: #/cm2 in standard pressure of 1e5
                mixf(region)%field(i,j,mixf(region)%ibuf) = &
                     fac*sum(rm(i,j,1:lm(region),itracer_out))/dxyp(j) !*1e5/p(i,j)
                mixf(region)%pso(i,j,mixf(region)%ibuf) = p(i,j,1)
                !                        rm(i,j,1,itracer_out) / &  ! mixing ratio
                !                        m(i,j,1)*fscale(itracer_out)
             end do
          end do

          if ( okdebug ) print *, 'output_mix: Mix output produced for region:',&
               region , ' buffer = ', mixf(region)%ibuf 

       end if

       nullify(rm)    ! reset pointers
       nullify(m)    ! reset pointers
       nullify(p)
       nullify(dxyp)
#ifdef MPI
       deallocate(rmg)
       deallocate(mg)
#endif

    end if   !skipper

  end subroutine output_mix


end module user_output_mix
